"""Baseline strings for test_delete_exif tests."""

from exif.tests.delete_exif_baselines.delete_ascii_tags_hex import DELETE_ASCII_TAGS_HEX_BASELINE
from exif.tests.delete_exif_baselines.delete_geotag_hex import DELETE_GEOTAG_HEX_BASELINE
