"""Baseline strings for test_modify_exif tests."""

from exif.tests.modify_exif_baselines.modify_ascii_same_len_hex import (
    MODIFY_ASCII_SAME_LEN_HEX_BASELINE)
from exif.tests.modify_exif_baselines.modify_ascii_shorter_hex import (
    MODIFY_ASCII_SHORTER_HEX_BASELINE)
from exif.tests.modify_exif_baselines.modify_orientation_hex import ROTATED_GRAND_CANYON_HEX
from exif.tests.modify_exif_baselines.modify_rational_hex import MODIFY_RATIONAL_HEX_BASELINE
from exif.tests.modify_exif_baselines.modify_srational_hex import MODIFY_SRATIONAL_HEX_BASELINE
from exif.tests.modify_exif_baselines.modify_user_comment import MODIFY_USER_COMMENT_BASELINE
